@@ -20,7 +20,10 @@ Created on 2020-10-29
 from wikibot.wikiclient import WikiClient
 from mwclient.image import Image
 from wikibot.smw import SMWClient
+#from difflib import Differ
+import difflib
 import os
+import re
 from pathlib import Path
 import sys
 from argparse import ArgumentParser
@@ -30,18 +33,21 @@ class WikiPush(object):
     '''
     Push pages from one MediaWiki to another
     '''
-
+    differ=None
+    
     def __init__(self, fromWikiId, toWikId,login=False,verbose=True,debug=False):
         '''
         Constructor
         '''
         self.verbose=verbose
         self.debug=debug
+        
         self.fromWikiId=fromWikiId
-        self.fromWiki=WikiClient.ofWikiId(fromWikiId)
+        if self.fromWikiId is not None:
+            self.fromWiki=WikiClient.ofWikiId(fromWikiId)
         self.toWikiId=toWikId
         self.toWiki=WikiClient.ofWikiId(toWikId)
-        if login:
+        if login and self.fromWikiId is not None:
             self.fromWiki.login()
         self.toWiki.login()
         
@@ -49,17 +55,20 @@ class WikiPush(object):
         if self.verbose:
                 print (msg,end=end)
      
-    def query(self,askQuery,queryField=None):
+    def query(self,askQuery,wiki=None,queryField=None):
         '''
-        query the from Wiki for pages matching the given askQuery
+        query the given wiki for pages matching the given askQuery
         
         Args:
             askQuery(string): Semantic Media Wiki in line query https://www.semantic-mediawiki.org/wiki/Help:Inline_queries
-            
+            wiki(wikibot): the wiki to query - use fromWiki if not specified
+            queryField(string): the field to select the pageTitle from
         Returns:
             list: a list of pageTitles matching the given askQuery
         '''
-        smwClient=SMWClient(self.fromWiki.getSite())
+        if wiki is None:
+            wiki=self.fromWiki
+        smwClient=SMWClient(wiki.getSite())
         pageRecords=smwClient.query(askQuery)  
         if queryField is None:
             return pageRecords.keys()
@@ -69,6 +78,98 @@ class WikiPush(object):
             pagesDict[pageRecord[queryField]]=True
         return pagesDict.keys()
     
+    def nuke(self,pageTitles,force=False):
+        '''
+        delete the pages with the given page Titles
+        
+        Args:
+            pageTitles(list): a list of page titles to be transfered from the formWiki to the toWiki
+            force(bool): True if pages should be actually deleted - dry run only listing pages is default
+        '''
+        total=len(pageTitles)
+        self.log("deleting %d pages in %s (%s)" % (total,self.toWikiId,"forced" if force else "dry run"))
+        for i,pageTitle in enumerate(pageTitles):
+            try:
+                self.log("%d/%d (%4.0f%%): deleting %s ..." % (i+1,total,(i+1)/total*100,pageTitle), end='')
+                pageToBeDeleted=self.toWiki.getPage(pageTitle)   
+                if not force:
+                    self.log("👍" if pageToBeDeleted.exists else "👎")
+                else:
+                    pageToBeDeleted.delete("deleted by wiknuke")    
+                    self.log("✅")
+            except Exception as ex:
+                msg=str(ex)
+                self.log("❌:%s" % msg )
+                
+    @staticmethod
+    def getDiff(text,newText,n=1,forHuman=True):
+        #if WikiPush.differ is None:
+        #    WikiPush.differ=Differ()
+        # https://docs.python.org/3/library/difflib.html
+        #  difflib.unified_diff(a, b, fromfile='', tofile='', fromfiledate='', tofiledate='', n=3, lineterm='\n')¶
+        #diffs=WikiPush.differ.compare(,)
+        textLines=text.split("\n")
+        newTextLines=newText.split("\n")
+        diffs=difflib.unified_diff(textLines,newTextLines,n=n)
+        if forHuman:
+            hdiffs=[]
+            for line in diffs:
+                unwantedItems=["@@","---","+++"]
+                keep=True
+                for unwanted in unwantedItems:
+                    if unwanted in line:
+                        keep=False
+                if keep:
+                    hdiffs.append(line)    
+        else:
+            hdiffs=diffs
+        diffStr="\n".join(hdiffs)
+        return diffStr
+    
+    @staticmethod
+    def getModify(search,replace):
+        searchRegex=r"%s" % search
+        replaceRegex=r"%s" % replace
+        modify=lambda text: re.sub(searchRegex,replaceRegex,text)
+        return modify
+
+                
+    def edit(self,pageTitles,modify=None,context=1,force=False):
+        '''
+        edit the pages with the given page Titles
+        
+        Args:
+            pageTitles(list): a list of page titles to be transfered from the formWiki to the toWiki
+            force(bool): True if pages should be actually deleted - dry run only listing pages is default
+        '''
+        if modify is None:
+            raise Exception("wikipush edit needs a modify function!")
+        total=len(pageTitles)
+        self.log("editing %d pages in %s (%s)" % (total,self.toWikiId,"forced" if force else "dry run"))
+        for i,pageTitle in enumerate(pageTitles):
+            try:
+                self.log("%d/%d (%4.0f%%): editing %s ..." % (i+1,total,(i+1)/total*100,pageTitle), end='')
+                pageToBeEdited=self.toWiki.getPage(pageTitle)   
+                if not force and not pageToBeEdited.exists:
+                    self.log("👎")
+                else:
+                    comment="edited by wikiedit" 
+                    text=pageToBeEdited.text()
+                    newText=modify(text)
+                    if newText!=text:
+                        if force:
+                            pageToBeEdited.edit(newText,comment)  
+                            self.log("✅")
+                        else:
+                            diffStr=self.getDiff(text, newText,n=context)
+                            self.log("👍%s" % diffStr)
+                    else:
+                        self.log("↔")
+            except Exception as ex:
+                msg=str(ex)
+                self.log("❌:%s" % msg )            
+                
+    
     def push(self,pageTitles,force=False,ignore=False,withImages=False):
         '''
         push the given page titles
@@ -92,7 +193,8 @@ class WikiPush(object):
                     else:
                         newPage=self.toWiki.getPage(pageTitle)
                         if not newPage.exists or force:
-                            newPage.edit(page.text(),"pushed by wikipush")
+                            comment="pushed from %s by wikipush" % self.fromWikiId
+                            newPage.edit(page.text(),comment)
                             self.log("✅")
                             if withImages:
                                 self.pushImages(page.images(),ignore=ignore)
@@ -155,7 +257,16 @@ __date__ = '2020-10-31'
 __updated__ = '2020-10-31'
 DEBUG=False
 
-def main(argv=None): # IGNORE:C0111
+def mainNuke(argv=None):
+    main(argv,mode='wikinuke')
+    
+def mainEdit(argv=None):
+    main(argv,mode='wikiedit')
+    
+def mainPush(argv=None):
+    main(argv,mode='wikipush')
+    
+def main(argv=None,mode='wikipush'): # IGNORE:C0111
     '''main program.'''
 
     if argv is None:
@@ -163,7 +274,7 @@ def main(argv=None): # IGNORE:C0111
     else:
         sys.argv.extend(argv)   
     
-    program_name = os.path.basename(sys.argv[0]) 
+    program_name = mode
     program_version = "v%s" % __version__
     program_build_date = str(__updated__)
     program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
@@ -189,26 +300,48 @@ USAGE
         parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
         parser.add_argument("-d", "--debug", dest="debug",   action="count", help="set debug level [default: %(default)s]")
         parser.add_argument('-V', '--version', action='version', version=program_version_message)
-        parser.add_argument("-l", "--login", dest="login", action='store_true', help="login to source wiki for access permission")
-        parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to overwrite existing pages")
-        parser.add_argument("-i", "--ignore", dest="ignore", action='store_true', help="ignore upload warnings e.g. duplicate images")
-        parser.add_argument("-wi", "--withImages", dest="withImages", action='store_true', help="copy images on the given pages")
+        if mode=="wikipush":
+            parser.add_argument("-l", "--login", dest="login", action='store_true', help="login to source wiki for access permission")
+            parser.add_argument("-s", "--source", dest="source", help="source wiki id", required=True)
+            parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to overwrite existing pages")
+            parser.add_argument("-i", "--ignore", dest="ignore", action='store_true', help="ignore upload warnings e.g. duplicate images")
+            parser.add_argument("-wi", "--withImages", dest="withImages", action='store_true', help="copy images on the given pages")
+        elif mode=="wikinuke":
+            parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to delete pages - default is 'dry' run only listing pages")            
+        elif mode=="wikiedit":
+            parser.add_argument("--search", dest="search", help="search pattern", required=True)
+            parser.add_argument("--replace", dest="replace", help="replace pattern", required=True)
+            parser.add_argument("--context", dest="context",type=int, help="number of context lines to show in dry run diff display",default=1)
+            parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to edit pages - default is 'dry' run only listing pages")            
         parser.add_argument("-q", "--query", dest="query", help="select pages with given SMW ask query", required=False)
         parser.add_argument("-qf", "--queryField",dest="queryField",help="query result field which contains page")
         
-        parser.add_argument("-s", "--source", dest="source", help="source wiki id", required=True)
         parser.add_argument("-t", "--target", dest="target", help="target wiki id", required=True)    
         parser.add_argument("-p", "--pages", nargs='+', help="list of page Titles to be pushed", required=False)
         # Process arguments
         args = parser.parse_args()
         
-        wikipush=WikiPush(args.source,args.target,login=args.login)
+        if mode=="wikipush":
+            wikipush=WikiPush(args.source,args.target,login=args.login)
+            queryWiki=wikipush.fromWiki
+        else:
+            wikipush=WikiPush(None,args.target)
+            queryWiki=wikipush.toWiki
         if args.pages:
             pages=args.pages
         elif args.query:
-            pages=wikipush.query(args.query,args.queryField)
+            pages=wikipush.query(args.query,wiki=queryWiki,queryField=args.queryField)
         if pages:
-            wikipush.push(pages,force=args.force,ignore=args.ignore,withImages=args.withImages)
+            if mode=="wikipush":
+                wikipush.push(pages,force=args.force,ignore=args.ignore,withImages=args.withImages)
+            elif mode=='wikinuke':
+                wikipush.nuke(pages,force=args.force)
+            elif mode=='wikiedit':
+                modify=WikiPush.getModify(args.search,args.replace)
+                wikipush.edit(pages,modify=modify,context=args.context,force=args.force)
+            else:
+                raise Exception("undefined wikipush mode %s" % mode)
+      
         
     except KeyboardInterrupt:
         ### handle keyboard interrupt ###
@@ -224,4 +357,4 @@ USAGE
 if __name__ == "__main__":
     if DEBUG:
         sys.argv.append("-d")
-    sys.exit(main())
\ No newline at end of file
+    sys.exit(mainPush())
\ No newline at end of file
